import csv
import re
from typing import List, Type

from typing_extensions import Union

from IsarnConnect.Meter import Meter
from IsarnConnect.RowArray import RowArray
from IsarnConnect.db.ELS import ELS
from IsarnConnect.db.LGZ import LGZ
from IsarnConnect.db.SLB import SLB
from IsarnConnect.meters import Row


def ablToRow(path):
    with open(path) as file:
        readfile = file.read()
        datalist = []

        while re.search("\d\.\d\.\d\(\d+\.?\d*(\*[a-zA-Z]*)?\)", readfile):
            span = re.search("\d\.\d\.\d\(\d+\.?\d*(\*[a-zA-Z]*)?\)", readfile).span()
            match = re.search("\d\.\d\.\d\(\d+\.?\d*(\*[a-zA-Z]*)?\)", readfile).group()
            datalist.append(match)
            readfile = readfile[span[1]:]

        entries = []

        for string in datalist:
            entry = float(
                string[6:-1].replace("*kW", "").replace("h", "").replace("*kvarh", "").replace("*kvar", "").replace(
                    "*V", "").replace("*A", ""))
            entries.append(entry)

        meter = file.name.split("/")[-1][1:4]
        if meter == "ELS":
            row = Row.ELS(entries[0], entries[1], entries[2], entries[3], entries[4], entries[5], entries[6],
                          entries[7], entries[8], entries[9], entries[10], entries[11], entries[12], entries[13],
                          entries[14], entries[15], entries[16])
            return row
        elif meter == "LGZ":
            row = Row.LGZ(entries[0], entries[1], entries[2], entries[3], entries[4], entries[5], entries[6],
                          entries[7], entries[8], entries[9], entries[10], entries[11], entries[12], entries[13],
                          entries[14], entries[15], entries[16], entries[17], entries[18], entries[19], entries[20])
            return row
        elif meter == "SLB":
            row = Row.SLB(entries[0], entries[1], entries[2], entries[3], entries[4], entries[5], entries[6],
                          entries[7], entries[8], entries[9], entries[10], entries[11], entries[12], entries[13],
                          entries[14], entries[15], entries[16], entries[17], entries[18], entries[19], entries[20],
                          entries[21], entries[22], entries[23], entries[24], entries[25], entries[26], entries[27],
                          entries[28], entries[29])
            return row


def csvToRowArray(path, meter: Meter):
    with open(path) as file:
        reader = csv.DictReader(file)
        array = RowArray(meter)
        if meter.value == "els":
            for row in reader:
                array_row = Row.ELS(c091=float(row['0.9.1']), c092=float(row['0.9.2']), c010=float(row['0.1.0']),
                                    c120=float(row['1.2.0'].replace("*kW", "")),
                                    c150=float(row['1.5.0'].replace("*kW", "")), c160=float(row['1.6.0']),
                                    c220=float(row['2.2.0'].replace("*kW", "")),
                                    c250=float(row['2.5.0'].replace("*kW", "")), c260=float(row['2.6.0']),
                                    c180=float(row['1.8.0']), c280=float(row['2.8.0']), c580=float(row['5.8.0']),
                                    c680=float(row['6.8.0']), c780=float(row['7.8.0'].replace("None", "00000.000")),
                                    c880=float(row['8.8.0'].replace("None", "00000.000")),
                                    c030=float(row['0.3.0'].replace("None", "00000.000")),
                                    c033=float(row['0.3.3'].replace("None", "00000.000")))
                array.append(array_row)
            return array
        elif meter.value == "lgz":
            for row in reader:
                array_row = Row.LGZ(c091=float(row['0.9.1']), c092=float(row['0.9.2']), c010=float(row['0.1.0']),
                                    c150=float(row['1.5.0'].replace("*kW", "")),
                                    c160=float(row['1.6.0'].replace("*kW", "")),
                                    c180=float(row['1.8.0'].replace("*kWh", "")),
                                    c250=float(row['2.5.0'].replace("*kW", "")),
                                    c260=float(row['2.6.0'].replace("*kW", "")),
                                    c280=float(row['2.8.0'].replace("*kWh", "")),
                                    c580=float(row['5.8.0'].replace("*kvarh", "")),
                                    c680=float(row['6.8.0'].replace("*kvarh", "")),
                                    c780=float(row['7.8.0'].replace("*kvarh", "")),
                                    c880=float(row['8.8.0'].replace("*kvarh", "")),
                                    c021=float(row['0.2.1']),
                                    c2700=float(row['2.7.0'].replace("*V", "")),
                                    c2701=float(row['2.7.0'].replace("*V", "")),
                                    c2702=float(row['2.7.0'].replace("*V", "")),
                                    c1700=float(row['1.7.0'].replace("*A", "")),
                                    c1701=float(row['1.7.0'].replace("*A", "")),
                                    c1702=float(row['1.7.0'].replace("*A", "")),
                                    c512=float(row['5.1.2'].replace("*kW", "")))
                array.append(array_row)
            return array
        elif meter.value == "slb":
            for row in reader:
                array_row = Row.SLB(c000=float(row['0.0.0']), c001=float(row['0.0.1']), c002=float(row['0.0.2']),
                                    c003=float(row['0.0.3']), c010=float(row['0.1.0']), c091=float(row['0.9.1']),
                                    c092=float(row['0.9.2']), c121=float(row['1.2.1']), c122=float(row['1.2.2']),
                                    c141=float(row['1.4.1']), c142=float(row['1.4.2']), c151=float(row['1.5.1']),
                                    c152=float(row['1.5.2']), c161=float(row['1.6.1']), c162=float(row['1.6.2']),
                                    c180=float(row['1.8.0']), c181=float(row['1.8.1']), c182=float(row['1.8.2']),
                                    c183=float(row['1.8.3']), c184=float(row['1.8.4']), c580=float(row['5.8.0']),
                                    c581=float(row['5.8.1']), c582=float(row['5.8.2']), c583=float(row['5.8.3']),
                                    c584=float(row['5.8.4']), c880=float(row['8.8.0']), c881=float(row['8.8.1']),
                                    c882=float(row['8.8.2']), c883=float(row['8.8.3']), c884=float(row['8.8.4']))
                array.append(array_row)
            return array


def tableRowToRowArray(rows: Union[List[Type[ELS]], List[Type[LGZ]], List[Type[SLB]]], meter: Meter):
    array = RowArray(meter)
    for row in rows:
        array.append(row.toRow())
    return array


from datetime import datetime

from pandas import DataFrame
from sqlalchemy import create_engine, desc
from sqlalchemy.orm import sessionmaker

from IsarnConnect.DatabaseType import DatabaseType
from IsarnConnect.Meter import Meter
from IsarnConnect.RowArray import RowArray
from IsarnConnect.db.ELS import Base as ELSBase, ELS
from IsarnConnect.db.LGZ import Base as LGZBase, LGZ
from IsarnConnect.db.SLB import Base as SLBBase, SLB

from typing import Union, List, Type

from IsarnConnect.meters import Row
from IsarnConnect.methods import tableRowToRowArray


class IsarnConnection:
    def __init__(self, password, username, address, port, database, database_type):
        self.username = username
        self.password = password
        self.address = address
        self.port = port
        self.database = database
        if database_type == DatabaseType.POSTGRE:
            self.engine = create_engine(
                'postgresql+psycopg2://' + username + ':' + password + '@' + address + ':' + port +
                '/' + database, echo=False)
        elif database_type == DatabaseType.MYSQL:
            self.engine = create_engine(
                'mysql+mysqlconnector://' + username + ':' + password + '@' + address + ':' + port +
                '/' + database, echo=False)
        self.session = sessionmaker(bind=self.engine)

    def connect(self):
        ELSBase.metadata.create_all(self.engine, checkfirst=True)
        LGZBase.metadata.create_all(self.engine, checkfirst=True)
        SLBBase.metadata.create_all(self.engine, checkfirst=True)

    def commitOnly(self, row: Union[Row.ELS, Row.LGZ, Row.SLB]):
        session = self.session()
        if isinstance(row, Row.ELS):
            session.add(row.toTableRow())
        elif isinstance(row, Row.LGZ):
            session.add(row.toTableRow())
        elif isinstance(row, Row.SLB):
            session.add(row.toTableRow())
        session.commit()

    def commitAll(self, rows: RowArray):
        session = self.session()
        if rows.meter.value == "els":
            print("els!")
            session.add_all(rows)
        elif rows.meter.value == "lgz":
            print("lgz!")
            session.add_all(rows)
        elif rows.meter.value == "slb":
            print("slb!")
            session.add_all(rows)
        session.commit()

    def getByDate(self, date: datetime, meter: Meter):
        session = self.session()
        date_code = float(date.strftime("%Y%m%d")[2:])
        if meter.value == "els":
            items: List[Type[ELS]] = session.query(ELS).filter_by(c092=date_code).all()
            return tableRowToRowArray(items, Meter.ELS)
        elif meter.value == "lgz":
            items: List[Type[LGZ]] = session.query(LGZ).filter_by(c092=date_code).all()
            return tableRowToRowArray(items, Meter.LGZ)
        elif meter.value == "slb":
            items: List[Type[SLB]] = session.query(SLB).filter_by(c092=date_code).all()
            return tableRowToRowArray(items, Meter.SLB)

    def getByTime(self, time: datetime, meter: Meter):
        session = self.session()
        time_code = float(time.strftime("%H%M%S"))
        if meter.value == "els":
            items = session.query(ELS).filter_by(c091=time_code).all()
            return tableRowToRowArray(items, Meter.ELS)
        elif meter.value == "lgz":
            items = session.query(LGZ).filter_by(c091=time_code).all()
            return tableRowToRowArray(items, Meter.LGZ)
        elif meter.value == "slb":
            items = session.query(SLB).filter_by(c091=time_code).all()
            return tableRowToRowArray(items, Meter.SLB)

    def getByDateTime(self, datetime: datetime, meter: Meter):
        session = self.session()
        condition = (
                (ELS.c092 == 190918) &
                (ELS.c091 == 115759)
        )
        if meter.value == "els":
            item = session.query(ELS).filter(condition).order_by(ELS.c092, ELS.c091).first()
            return item.toRow()
        elif meter.value == "lgz":
            item = session.query(LGZ).filter(condition).order_by(LGZ.c092, LGZ.c091).first()
            return item.toRow()
        elif meter.value == "slb":
            item = session.query(SLB).filter(condition).order_by(SLB.c092, SLB.c091).first()
            return item.toRow()

    def getBetweenDatetime(self, from_date: datetime, to_date: datetime, meter: Meter):
        session = self.session()
        from_date_code = float(from_date.strftime("%Y%m%d")[2:] + "" + from_date.strftime("%H%M%S"))
        to_date_code = float(to_date.strftime("%Y%m%d")[2:] + "" + to_date.strftime("%H%M%S"))
        if meter.value == "els":
            items = (session
                     .query(ELS).filter((ELS.c092 * 1000000 + ELS.c091).between(from_date_code, to_date_code))
                     .order_by(ELS.c092, ELS.c091)
                     .all())
            return tableRowToRowArray(items, Meter.ELS)
        elif meter.value == "lgz":
            items = (session
                     .query(LGZ).filter((ELS.c092 * 1000000 + ELS.c091).between(from_date_code, to_date_code))
                     .order_by(ELS.c092, ELS.c091)
                     .all())
            return tableRowToRowArray(items, Meter.LGZ)
        elif meter.value == "slb":
            items = (session
                     .query(SLB).filter((ELS.c092 * 1000000 + ELS.c091).between(from_date_code, to_date_code))
                     .order_by(ELS.c092, ELS.c091)
                     .all())
            return tableRowToRowArray(items, Meter.SLB)

    def getBetweenTime(self, from_time: datetime, to_time: datetime, meter: Meter):
        session = self.session()
        from_time_code = float(from_time.strftime("%H%M%S"))
        to_time_code = float(to_time.strftime("%H%M%S"))

        if meter.value == "els":
            items = session.query(ELS).filter(ELS.c091.between(from_time_code, to_time_code)).all()
            return tableRowToRowArray(items, Meter.ELS)
        elif meter.value == "lgz":
            items = session.query(LGZ).filter(LGZ.c091.between(from_time_code, to_time_code)).all()
            return tableRowToRowArray(items, Meter.LGZ)
        elif meter.value == "slb":
            items = session.query(SLB).filter(SLB.c091.between(from_time_code, to_time_code)).all()
            return tableRowToRowArray(items, Meter.SLB)

    def getLatest(self, meter: Meter):
        session = self.session()
        if meter.value == "els":
            item = session.query(ELS).order_by(desc(ELS.id)).first()
            return item.toRow()
        elif meter.value == "lgz":
            item = session.query(LGZ).order_by(desc(LGZ.id)).first()
            return item
        elif meter.value == "slb":
            item = session.query(SLB).order_by(desc(SLB.id)).first()
            return item

    def getCombinedForDatetime(self, from_date: datetime, to_date: datetime):
        session = self.session()
        from_date_code = float(from_date.strftime("%Y%m%d")[2:] + "" + from_date.strftime("%H%M%S"))
        to_date_code = float(to_date.strftime("%Y%m%d")[2:] + "" + to_date.strftime("%H%M%S"))
        itemsELS = (session
                    .query(ELS).filter((ELS.c092 * 1000000 + ELS.c091).between(from_date_code, to_date_code))
                    .order_by(ELS.c092, ELS.c091)
                    .all())
        itemsELSDF = DataFrame(tableRowToRowArray(itemsELS, Meter.ELS))
        elsUse = 0
        if not itemsELSDF.empty:
            elsUse = itemsELSDF.iloc[-1][0].c180 - itemsELSDF.iloc[0][0].c180
        itemsLGZ = (session
                    .query(LGZ).filter((LGZ.c092 * 1000000 + LGZ.c091).between(from_date_code, to_date_code))
                    .order_by(LGZ.c092, LGZ.c091)
                    .all())
        itemsLGZDF = DataFrame(tableRowToRowArray(itemsLGZ, Meter.LGZ))
        lgzuse = 0
        if not itemsLGZDF.empty:
            lgzuse = itemsLGZDF.iloc[-1][0].c180 - itemsLGZDF.iloc[0][0].c180
        itemsSLB = (session
                    .query(SLB).filter((SLB.c092 * 1000000 + SLB.c091).between(from_date_code, to_date_code))
                    .order_by(SLB.c092, SLB.c091)
                    .all())
        itemsSLBDF = DataFrame(tableRowToRowArray(itemsSLB, Meter.SLB))
        slbuse = 0
        if not itemsSLBDF.empty:
            slbuse = itemsSLBDF.iloc[-1][0].c180 - itemsSLBDF.iloc[0][0].c180
        return elsUse + lgzuse + slbuse



from typing import Union, List

from IsarnConnect.Meter import Meter
from IsarnConnect.meters import Row


class RowArray:
    def __init__(self, meter: Meter):
        self.elements = []
        self.meter = meter

    def addList(self, list: Union[List[Row.ELS], List[Row.SLB], List[Row.LGZ]]):
        for row in list:
            self.elements.append(row.toTableRow())

    def append(self, row: Union[Row.ELS, Row.LGZ, Row.SLB]):
        if isinstance(row, Row.ELS) and self.meter.value == "els":
            self.elements.append(row.toTableRow())
        elif isinstance(row, Row.LGZ) and self.meter == "lgz":
            self.elements.append(row.toTableRow())
        elif isinstance(row, Row.SLB) and self.meter == "slb":
            self.elements.append(row.toTableRow())

    def pop(self, index):
        self.elements.pop(index)

    def remove(self, row):
        self.elements.remove(row.toTableRow())

    def __str__(self):
        return str(self.elements)

    def __iter__(self):
        return iter(self.elements)

    def asList(self):
        list = []
        for row in self.elements:
            list.append(row.toRow())
        return list



from sqlalchemy import Double, Integer
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import mapped_column

from IsarnConnect.meters import Row

Base = declarative_base()


class SLB(Base):
    __tablename__ = 'SLB_data'
    __table_args__ = {'schema': 'test'} # TODO reorganize Development Stage!
    id = mapped_column("id", Integer, autoincrement=True, primary_key=True)
    c000 = mapped_column("0.0.0", Double)
    c001 = mapped_column("0.0.1", Double)
    c002 = mapped_column("0.0.2", Double)
    c003 = mapped_column("0.0.3", Double)
    c010 = mapped_column("0.1.0", Double)
    c091 = mapped_column("0.9.1", Double)
    c092 = mapped_column("0.9.2", Double)
    c121 = mapped_column("1.2.1", Double)
    c122 = mapped_column("1.2.2", Double)
    c141 = mapped_column("1.4.1", Double)
    c142 = mapped_column("1.4.2", Double)
    c151 = mapped_column("1.5.1", Double)
    c152 = mapped_column("1.5.2", Double)
    c161 = mapped_column("1.6.1", Double)
    c162 = mapped_column("1.6.2", Double)
    c180 = mapped_column("1.8.0", Double)
    c181 = mapped_column("1.8.1", Double)
    c182 = mapped_column("1.8.2", Double)
    c183 = mapped_column("1.8.3", Double)
    c184 = mapped_column("1.8.4", Double)
    c580 = mapped_column("5.8.0", Double)
    c581 = mapped_column("5.8.1", Double)
    c582 = mapped_column("5.8.2", Double)
    c583 = mapped_column("5.8.3", Double)
    c584 = mapped_column("5.8.4", Double)
    c880 = mapped_column("8.8.0", Double)
    c881 = mapped_column("8.8.1", Double)
    c882 = mapped_column("8.8.2", Double)
    c883 = mapped_column("8.8.3", Double)
    c884 = mapped_column("8.8.4", Double)

    def __str__(self):
        return ({"c000": self.c000, "c001": self.c001, "c002": self.c002, "c003": self.c003, "c010": self.c010,
                "c091": self.c091, "c092": self.c092, "c121": self.c121, "c122": self.c122, "c141": self.c141,
                "c142": self.c142, "c151": self.c151, "c152": self.c152, "c161": self.c161, "c162": self.c162,
                "c180": self.c180, "c181": self.c181, "c182": self.c182, "c183": self.c183, "c184": self.c184,
                "c580": self.c580, "c581": self.c581, "c582": self.c582, "c583": self.c583, "c584": self.c584,
                "c880": self.c880, "c881": self.c881, "c882": self.c882, "c883": self.c883, "c884": self.c884}
                .__str__())

    def toRow(self):
        return Row.SLB(c000= self.c000, c001= self.c001, c002= self.c002, c003= self.c003, c010=self.c010,
                       c091= self.c091, c092= self.c092, c121= self.c121, c122= self.c122, c141=self.c141,
                       c142= self.c142, c151= self.c151, c152= self.c152, c161= self.c161, c162= self.c162,
                       c180= self.c180, c181= self.c181, c182= self.c182, c183= self.c183, c184= self.c184,
                       c580= self.c580, c581= self.c581, c582= self.c582, c583= self.c583, c584= self.c584,
                       c880= self.c880, c881= self.c881, c882= self.c882, c883= self.c883, c884= self.c884)


from setuptools import setup, find_packages

setup(
    name='IsarnConnect',
    version='1.4.1',
    author='C0MaE',
    author_email='jannis.koberg@icloud.com',
    packages=find_packages(),
    install_requires=[
        'psycopg2-binary>=2.9.9',
        'sqlalchemy>=2.0.27',
        'pandas>=2.2.1',
        'mysql-connector-python>=2.2.9'
    ],
    long_description=open('README.md', 'r').read(),
    long_description_content_type='text/markdown',
)



from sqlalchemy import Double, Integer
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import mapped_column

from IsarnConnect.meters import Row

Base = declarative_base()


class ELS(Base):
    __tablename__ = 'ELS_data'
    __table_args__ = {'schema': 'test'} # TODO reorganize Development Stage!
    id = mapped_column("id", Integer, autoincrement=True, primary_key=True)
    c091 = mapped_column("0.9.1", Double)
    c092 = mapped_column("0.9.2", Double)
    c010 = mapped_column("0.1.0", Double)
    c120 = mapped_column("1.2.0", Double)
    c150 = mapped_column("1.5.0", Double)
    c160 = mapped_column("1.6.0", Double)
    c220 = mapped_column("2.2.0", Double)
    c250 = mapped_column("2.5.0", Double)
    c260 = mapped_column("2.6.0", Double)
    c180 = mapped_column("1.8.0", Double)
    c280 = mapped_column("2.8.0", Double)
    c580 = mapped_column("5.9.0", Double)
    c680 = mapped_column("6.8.0", Double)
    c780 = mapped_column("7.8.0", Double)
    c880 = mapped_column("8.8.0", Double)
    c030 = mapped_column("0.3.0", Double)
    c033 = mapped_column("0.3.3", Double)

    def __str__(self):
        return {"c091": self.c091, "c092": self.c092, "c010": self.c010, "c120": self.c120, "c150": self.c150,
                "c160": self.c160, "c220": self.c220, "c250": self.c250, "c260": self.c260, "c180": self.c180,
                "c280": self.c280, "c580": self.c580, "c680": self.c680, "c780": self.c780, "c880": self.c880,
                "c030": self.c030, "c033": self.c033}.__str__()

    def toRow(self):
        return Row.ELS(c091=self.c091, c092=self.c092, c010=self.c010, c120=self.c120, c150=self.c150, c160=self.c160,
                       c220=self.c220, c250=self.c250, c260=self.c260, c180=self.c180*60, c280=self.c280*60, c580=self.c580,
                       c680=self.c680, c780=self.c780, c880=self.c880, c030=self.c030, c033=self.c033)